package javax.swing;

import java.beans.Transient;
import java.util.EventListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.EventListenerList;
import java.util.BitSet;
import java.io.Serializable;

public class DefaultListSelectionModel implements ListSelectionModel, Cloneable, Serializable
{
    private static final int MIN = -1;
    private static final int MAX = Integer.MAX_VALUE;
    private int selectionMode;
    private int minIndex;
    private int maxIndex;
    private int anchorIndex;
    private int leadIndex;
    private int firstAdjustedIndex;
    private int lastAdjustedIndex;
    private boolean isAdjusting;
    private int firstChangedIndex;
    private int lastChangedIndex;
    private BitSet value;
    protected EventListenerList listenerList;
    protected boolean leadAnchorNotificationEnabled;
    
    public DefaultListSelectionModel() {
        this.selectionMode = 2;
        this.minIndex = Integer.MAX_VALUE;
        this.maxIndex = -1;
        this.anchorIndex = -1;
        this.leadIndex = -1;
        this.firstAdjustedIndex = Integer.MAX_VALUE;
        this.lastAdjustedIndex = -1;
        this.isAdjusting = false;
        this.firstChangedIndex = Integer.MAX_VALUE;
        this.lastChangedIndex = -1;
        this.value = new BitSet(32);
        this.listenerList = new EventListenerList();
        this.leadAnchorNotificationEnabled = true;
    }
    
    @Override
    public int getMinSelectionIndex() {
        return this.isSelectionEmpty() ? -1 : this.minIndex;
    }
    
    @Override
    public int getMaxSelectionIndex() {
        return this.maxIndex;
    }
    
    @Override
    public boolean getValueIsAdjusting() {
        return this.isAdjusting;
    }
    
    @Override
    public int getSelectionMode() {
        return this.selectionMode;
    }
    
    @Override
    public void setSelectionMode(int selectionMode) {
        switch (selectionMode) {
            case 0:
            case 1:
            case 2: {
                this.selectionMode = selectionMode;
                return;
            }
            default: {
                throw new IllegalArgumentException("invalid selectionMode");
            }
        }
    }
    
    @Override
    public boolean isSelectedIndex(int bitIndex) {
        return bitIndex >= this.minIndex && bitIndex <= this.maxIndex && this.value.get(bitIndex);
    }
    
    @Override
    public boolean isSelectionEmpty() {
        return this.minIndex > this.maxIndex;
    }
    
    @Override
    public void addListSelectionListener(ListSelectionListener l) {
        this.listenerList.add(ListSelectionListener.class, l);
    }
    
    @Override
    public void removeListSelectionListener(ListSelectionListener l) {
        this.listenerList.remove(ListSelectionListener.class, l);
    }
    
    public ListSelectionListener[] getListSelectionListeners() {
        return (ListSelectionListener[])this.listenerList.getListeners(ListSelectionListener.class);
    }
    
    protected void fireValueChanged(boolean b) {
        if (this.lastChangedIndex == -1) {
            return;
        }
        int firstChangedIndex = this.firstChangedIndex;
        int lastChangedIndex = this.lastChangedIndex;
        this.firstChangedIndex = Integer.MAX_VALUE;
        this.lastChangedIndex = -1;
        this.fireValueChanged(firstChangedIndex, lastChangedIndex, b);
        return;
    }
    
    protected void fireValueChanged(int n, int n2) {
        this.fireValueChanged(n, n2, this.getValueIsAdjusting());
    }
    
    protected void fireValueChanged(int firstIndex, int lastIndex, boolean isAdjusting) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListSelectionListener.class) {
                if (o == null) {
                    o = new ListSelectionEvent(this, firstIndex, lastIndex, isAdjusting);
                }
                ((ListSelectionListener)listenerList[i + 1]).valueChanged(o);
            }
        }
    }
    
    private void fireValueChanged() {
        if (this.lastAdjustedIndex == -1) {
            return;
        }
        if (this.getValueIsAdjusting()) {
            this.firstChangedIndex = Math.min(this.firstChangedIndex, this.firstAdjustedIndex);
            this.lastChangedIndex = Math.max(this.lastChangedIndex, this.lastAdjustedIndex);
        }
        int firstAdjustedIndex = this.firstAdjustedIndex;
        int lastAdjustedIndex = this.lastAdjustedIndex;
        this.firstAdjustedIndex = Integer.MAX_VALUE;
        this.lastAdjustedIndex = -1;
        this.fireValueChanged(firstAdjustedIndex, lastAdjustedIndex);
        return;
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
    
    private void markAsDirty(int n) {
        if (n == -1) {
            return;
        }
        this.firstAdjustedIndex = Math.min(this.firstAdjustedIndex, n);
        this.lastAdjustedIndex = Math.max(this.lastAdjustedIndex, n);
        return;
    }
    
    private void set(int n) {
        if (this.value.get(n)) {
            return;
        }
        this.value.set(n);
        this.markAsDirty(n);
        this.minIndex = Math.min(this.minIndex, n);
        this.maxIndex = Math.max(this.maxIndex, n);
        return;
    }
    
    private void clear(int n) {
        if (!this.value.get(n)) {
            return;
        }
        this.value.clear(n);
        this.markAsDirty(n);
        if (n == this.minIndex) {
            ++this.minIndex;
            while (this.minIndex <= this.maxIndex) {
                if (this.value.get(this.minIndex)) {
                    break;
                }
                ++this.minIndex;
            }
        }
        if (n == this.maxIndex) {
            --this.maxIndex;
            while (this.minIndex <= this.maxIndex) {
                if (this.value.get(this.maxIndex)) {
                    break;
                }
                --this.maxIndex;
            }
        }
        if (this.isSelectionEmpty()) {
            this.minIndex = Integer.MAX_VALUE;
            this.maxIndex = -1;
        }
        return;
    }
    
    public void setLeadAnchorNotificationEnabled(boolean leadAnchorNotificationEnabled) {
        this.leadAnchorNotificationEnabled = leadAnchorNotificationEnabled;
    }
    
    public boolean isLeadAnchorNotificationEnabled() {
        return this.leadAnchorNotificationEnabled;
    }
    
    private void updateLeadAnchorIndices(int anchorIndex, int leadIndex) {
        if (this.leadAnchorNotificationEnabled) {
            if (this.anchorIndex != anchorIndex) {
                this.markAsDirty(this.anchorIndex);
                this.markAsDirty(anchorIndex);
            }
            if (this.leadIndex != leadIndex) {
                this.markAsDirty(this.leadIndex);
                this.markAsDirty(leadIndex);
            }
        }
        this.anchorIndex = anchorIndex;
        this.leadIndex = leadIndex;
    }
    
    private boolean contains(int n, int n2, int n3) {
        return n3 >= n && n3 <= n2;
    }
    
    private void changeSelection(int b, int b2, int a, int a2, boolean b3) {
        for (int i = Math.min(a, b); i <= Math.max(a2, b2); ++i) {
            int contains = this.contains(b, b2, i);
            int contains2 = this.contains(a, a2, i);
            if (contains2 != 0 && contains != 0) {
                if (b3) {
                    contains = 0;
                }
                else {
                    contains2 = 0;
                }
            }
            if (contains2 != 0) {
                this.set(i);
            }
            if (contains != 0) {
                this.clear(i);
            }
        }
        this.fireValueChanged();
    }
    
    private void changeSelection(int n, int n2, int n3, int n4) {
        this.changeSelection(n, n2, n3, n4, true);
    }
    
    @Override
    public void clearSelection() {
        this.removeSelectionIntervalImpl(this.minIndex, this.maxIndex, false);
    }
    
    @Override
    public void setSelectionInterval(int n, int n2) {
        if (n == -1 || n2 == -1) {
            return;
        }
        if (this.getSelectionMode() == 0) {
            n = n2;
        }
        this.updateLeadAnchorIndices(n, n2);
        this.changeSelection(this.minIndex, this.maxIndex, Math.min(n, n2), Math.max(n, n2));
        return;
    }
    
    @Override
    public void addSelectionInterval(int n, int n2) {
        if (n == -1 || n2 == -1) {
            return;
        }
        if (this.getSelectionMode() == 0) {
            this.setSelectionInterval(n, n2);
            return;
        }
        this.updateLeadAnchorIndices(n, n2);
        int n3 = Integer.MAX_VALUE;
        int n4 = -1;
        int min = Math.min(n, n2);
        int max = Math.max(n, n2);
        if (this.getSelectionMode() == 1 && (max < this.minIndex - 1 || min > this.maxIndex + 1)) {
            this.setSelectionInterval(n, n2);
            return;
        }
        this.changeSelection(n3, n4, min, max);
        return;
    }
    
    @Override
    public void removeSelectionInterval(int n, int n2) {
        this.removeSelectionIntervalImpl(n, n2, true);
    }
    
    private void removeSelectionIntervalImpl(int n, int n2, boolean b) {
        if (n == -1 || n2 == -1) {
            return;
        }
        if (b) {
            this.updateLeadAnchorIndices(n, n2);
        }
        int min = Math.min(n, n2);
        int n3 = Math.max(n, n2);
        int n4 = Integer.MAX_VALUE;
        int n5 = -1;
        if (this.getSelectionMode() != 2 && min > this.minIndex && n3 < this.maxIndex) {
            n3 = this.maxIndex;
        }
        this.changeSelection(min, n3, n4, n5);
        return;
    }
    
    private void setState(int n, boolean b) {
        if (b) {
            this.set(n);
        }
        else {
            this.clear(n);
        }
    }
    
    @Override
    public void insertIndexInterval(int bitIndex, int n, boolean b) {
        int i = b ? bitIndex : (bitIndex + 1);
        int n2 = i + n - 1;
        for (int j = this.maxIndex; j >= i; --j) {
            this.setState(j + n, this.value.get(j));
        }
        int n3 = this.getSelectionMode() != 0 && this.value.get(bitIndex);
        for (; i <= n2; ++i) {
            this.setState(i, (boolean)n3);
        }
        int leadIndex = this.leadIndex;
        if (leadIndex > bitIndex || (b && leadIndex == bitIndex)) {
            leadIndex = this.leadIndex + n;
        }
        int anchorIndex = this.anchorIndex;
        if (anchorIndex > bitIndex || (b && anchorIndex == bitIndex)) {
            anchorIndex = this.anchorIndex + n;
        }
        if (leadIndex != this.leadIndex || anchorIndex != this.anchorIndex) {
            this.updateLeadAnchorIndices(anchorIndex, leadIndex);
        }
        this.fireValueChanged();
    }
    
    @Override
    public void removeIndexInterval(int n, int n2) {
        int i = Math.min(n, n2);
        int max = Math.max(n, n2);
        int n3 = max - i + 1;
        for (; i <= this.maxIndex; ++i) {
            this.setState(i, this.value.get(i + n3));
        }
        int leadIndex = this.leadIndex;
        if (leadIndex != 0 || i != 0) {
            if (leadIndex > max) {
                leadIndex = this.leadIndex - n3;
            }
            else if (leadIndex >= i) {
                leadIndex = i - 1;
            }
        }
        int anchorIndex = this.anchorIndex;
        if (anchorIndex != 0 || i != 0) {
            if (anchorIndex > max) {
                anchorIndex = this.anchorIndex - n3;
            }
            else if (anchorIndex >= i) {
                anchorIndex = i - 1;
            }
        }
        if (leadIndex != this.leadIndex || anchorIndex != this.anchorIndex) {
            this.updateLeadAnchorIndices(anchorIndex, leadIndex);
        }
        this.fireValueChanged();
    }
    
    @Override
    public void setValueIsAdjusting(boolean isAdjusting) {
        if (isAdjusting != this.isAdjusting) {
            this.fireValueChanged(this.isAdjusting = isAdjusting);
        }
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + " " + Integer.toString(this.hashCode()) + " " + ((this.getValueIsAdjusting() ? "~" : "=") + this.value.toString());
    }
    
    public Object clone() throws CloneNotSupportedException {
        DefaultListSelectionModel defaultListSelectionModel = (DefaultListSelectionModel)super.clone();
        defaultListSelectionModel.value = (BitSet)this.value.clone();
        defaultListSelectionModel.listenerList = new EventListenerList();
        return defaultListSelectionModel;
    }
    
    @Transient
    @Override
    public int getAnchorSelectionIndex() {
        return this.anchorIndex;
    }
    
    @Transient
    @Override
    public int getLeadSelectionIndex() {
        return this.leadIndex;
    }
    
    @Override
    public void setAnchorSelectionIndex(int n) {
        this.updateLeadAnchorIndices(n, this.leadIndex);
        this.fireValueChanged();
    }
    
    public void moveLeadSelectionIndex(int n) {
        if (n == -1 && this.anchorIndex != -1) {
            return;
        }
        this.updateLeadAnchorIndices(this.anchorIndex, n);
        this.fireValueChanged();
        return;
    }
    
    @Override
    public void setLeadSelectionIndex(int b) {
        int anchorIndex = this.anchorIndex;
        if (b == -1) {
            if (anchorIndex == -1) {
                this.updateLeadAnchorIndices(anchorIndex, b);
                this.fireValueChanged();
            }
            return;
        }
        else {
            if (anchorIndex == -1) {
                return;
            }
            if (this.leadIndex == -1) {
                this.leadIndex = b;
            }
            int value = this.value.get(this.anchorIndex);
            if (this.getSelectionMode() == 0) {
                anchorIndex = b;
                value = true;
            }
            int min = Math.min(this.anchorIndex, this.leadIndex);
            int max = Math.max(this.anchorIndex, this.leadIndex);
            int min2 = Math.min(anchorIndex, b);
            int max2 = Math.max(anchorIndex, b);
            this.updateLeadAnchorIndices(anchorIndex, b);
            if (!(value == 0)) {
                this.changeSelection(min, max, min2, max2);
            }
            else {
                this.changeSelection(min2, max2, min, max, false);
            }
            return;
        }
    }
}
